<?php
require_once 'isunmailersubscriptionaggregator.interface.php';

require_once 'sunmailernewsletter.class.php';
require_once 'sunmailerdesign.class.php';
require_once 'sunmailersection.class.php';
require_once 'sunmailersubscriber.class.php';
require_once 'sunmailerschedule.class.php';
require_once 'sunmailersubscriptionaggregator.class.php';

/**
 * Sets the form element type based on which Date modules are enabled.
 */
function _sunmailer_set_date_type(&$element) {
  if (module_exists('date_api')) {
    $element['#type'] = module_exists('date_popup') ? 'date_popup' : 'date_select';
    $element['#format'] = 'Y-m-d H:i';
  }
  else {
    $element['#type'] = 'textfield';
    $element['#element_validate'] = array('_sunmailer_date_validate');
  }
}

/**
 * Validation callback for a date.
 */
function _sunmailer_date_validate($element, &$form_state) {
  if (!strtotime($element['#value'])) {
    form_error($element, t('Invalid date format.'));
  }
}

/**
 * Generates a link to access the subscription page.
 */
function sunmailer_subscription_link($mail = NULL, $nlids = array(), $sids = array())
{
  // Check for an anonymous link.
  if (!$mail) {
    return url('sunmailer/subscribe', array('absolute' => TRUE));
  }

  // Embed the mail and hash into the link.
  $subscriber = SunMailerSubscriber::loadSubscriber($mail);
  $query = array(
    'mail' => $subscriber->getMail(),
    'hash' => $subscriber->getHash(),
  );
  return url('sunmailer/subscribe', array('absolute' => TRUE, 'query' => $query));
}

/**
 * Sets up an instance of the PHPMailer class.
 * It still needs the email's contents and recipients.
 */
function sunmailer_get_mailer() {
  $path = variable_get('sunmailer_phpmailer_path', '');
  require_once $path ? "$path/class.phpmailer.php" : 'class.phpmailer.php';
  $mailer = new PHPMailer();
  $mailer->FromName = variable_get('sunmailer_from_name', '');
  $mailer->From = variable_get('sunmailer_from_email', '');
  $mailer->Host = variable_get('sunmailer_host', 'localhost');
  $mailer->Port = variable_get('sunmailer_port', 25);
  switch (variable_get('sunmailer_method', 'mail')) {
  case 'mail':
    $mailer->IsMail();
    break;
  case 'sendmail':
    $mailer->IsSendmail();
    $mailer->Sendmail = variable_get('sunmailer_sendmail_path', '/usr/bin/sendmail');
    break;
  case 'smtp':
    $mailer->IsSMTP();
    $mailer->SMTPAuth = variable_get('sunmailer_smtp_auth', FALSE);
    if ($mailer->SMTPAuth) {
      $mailer->Username = variable_get('sunmailer_smtp_user', '');
      $mailer->Password = variable_get('sunmailer_smtp_pass', '');
    }
    $mailer->SMTPSecure = variable_get('sunmailer_smtp_secure', '');
    break;
  }
  $mailer->CharSet = variable_get('sunmailer_char_set', 'utf-8');
  $mailer->WordWrap = variable_get('sunmailer_word_wrap', 80);
  return $mailer;
}

/**
 * Splits a list of recipients into groups for mailing.
 */
function sunmailer_split_addresses($addresses) {
  $group_size = variable_get('sunmailer_max_recipients', 1);
  $group_size = $group_size ? $group_size : count($addresses);
  $address_groups = array();
  for ($i = 0; $i < count($addresses); $i += $group_size) {
    $address_groups[] = array_slice($addresses, $i, $group_size);
  }
  return $address_groups;
}

/**
 * Sends an email.
 */
function sunmailer_send_mail($mailer, $addresses, $subject, $body, $is_html = TRUE, $alt_body = NULL) {
  // Set contents of email.
  $mailer->Subject = $subject;
  $mailer->Body = $body;
  $mailer->IsHTML($is_html);
  $mailer->AltBody = $is_html ? $alt_body : NULL;
  // Use To: for individual emails, Bcc: for group emails.
  $address_groups = sunmailer_split_addresses($addresses);
  foreach ($address_groups as $addresses) {
    $mailer->ClearAllRecipients();
    if (count($addresses) == 1) {
      $mailer->AddAddress(array_pop($addresses));
    }
    else {
      foreach ($addresses as $address) {
        $mailer->AddBCC($address);
      }
    }
    // Send mail.
    $retries = variable_get('sunmailer_retries', 2);
    while ($retries >= 0) {
      if ($mailer->Send()) {
        break;
      }
      $retries--;
      if ($retries >= 0) {
        watchdog('mail', 'Email failed to send (@error), retrying...', array('@error' => $mailer->ErrorInfo), WATCHDOG_WARNING);
      }
      else {
        watchdog('mail', 'Email failed to send (@error), giving up', array('@error' => $mailer->ErrorInfo), WATCHDOG_ERROR);
      }
    }
  }
}
